@@ -60,7 +60,6 @@ struct gatt_service {
 	GSList *primary;
 	GAttrib *attrib;
 	int psm;
-	guint atid;
 	gboolean listen;
 };
 
@@ -74,10 +73,8 @@ struct format {
 
 struct primary {
 	struct gatt_service *gatt;
+	struct att_primary *att;
 	char *path;
-	uuid_t uuid;
-	uint16_t start;
-	uint16_t end;
 	GSList *chars;
 	GSList *watchers;
 };
@@ -167,7 +164,7 @@ static int gatt_dev_cmp(gconstpointer a, gconstpointer b)
 	const struct gatt_service *gatt = a;
 	const struct btd_device *dev = b;
 
-	return gatt->dev == dev;
+	return gatt->dev != dev;
 }
 
 static int characteristic_handle_cmp(gconstpointer a, gconstpointer b)
@@ -191,18 +188,6 @@ static int watcher_cmp(gconstpointer a, gconstpointer b)
 	return g_strcmp0(watcher->path, match->path);
 }
 
-static inline DBusMessage *invalid_args(DBusMessage *msg)
-{
-	return g_dbus_create_error(msg, ERROR_INTERFACE ".InvalidArguments",
-			"Invalid arguments in method call");
-}
-
-static inline DBusMessage *not_authorized(DBusMessage *msg)
-{
-	return g_dbus_create_error(msg, ERROR_INTERFACE ".NotAuthorized",
-			"Not authorized");
-}
-
 static void append_char_dict(DBusMessageIter *iter, struct characteristic *chr)
 {
 	DBusMessageIter dict;
@@ -321,9 +306,6 @@ static void events_handler(const uint8_t *pdu, uint16_t len,
 	}
 }
 
-static void primary_cb(guint8 status, const guint8 *pdu, guint16 plen,
-							gpointer user_data);
-
 static void attrib_destroy(gpointer user_data)
 {
 	struct gatt_service *gatt = user_data;
@@ -342,7 +324,6 @@ static void attrib_disconnect(gpointer user_data)
 static void connect_cb(GIOChannel *chan, GError *gerr, gpointer user_data)
 {
 	struct gatt_service *gatt = user_data;
-	guint atid;
 
 	if (gerr) {
 		error("%s", gerr->message);
@@ -363,63 +344,11 @@ static void connect_cb(GIOChannel *chan, GError *gerr, gpointer user_data)
 		return;
 	}
 
-	atid = gatt_discover_primary(gatt->attrib, 0x0001, 0xffff, NULL,
-							primary_cb, gatt);
-	if (atid == 0)
-		goto fail;
-
-	gatt->atid = atid;
-
 	return;
 fail:
 	g_attrib_unref(gatt->attrib);
 }
 
-static DBusMessage *get_characteristics(DBusConnection *conn,
-						DBusMessage *msg, void *data)
-{
-	struct primary *prim = data;
-	DBusMessage *reply;
-	DBusMessageIter iter, array;
-	GSList *l;
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &iter);
-
-	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_OBJECT_PATH_AS_STRING
-			DBUS_TYPE_ARRAY_AS_STRING
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &array);
-
-	for (l = prim->chars; l; l = l->next) {
-		struct characteristic *chr = l->data;
-		DBusMessageIter sub;
-
-		DBG("path %s", chr->path);
-
-		dbus_message_iter_open_container(&array, DBUS_TYPE_DICT_ENTRY,
-								NULL, &sub);
-
-		dbus_message_iter_append_basic(&sub, DBUS_TYPE_OBJECT_PATH,
-								&chr->path);
-
-		append_char_dict(&sub, chr);
-
-		dbus_message_iter_close_container(&array, &sub);
-	}
-
-	dbus_message_iter_close_container(&iter, &array);
-
-	return reply;
-}
-
 static int l2cap_connect(struct gatt_service *gatt, GError **gerr,
 								gboolean listen)
 {
@@ -435,7 +364,15 @@ static int l2cap_connect(struct gatt_service *gatt, GError **gerr,
 	 * Configuration it is necessary to poll the server from time
 	 * to time checking for modifications.
 	 */
-	io = bt_io_connect(BT_IO_L2CAP, connect_cb, gatt, NULL, gerr,
+	if (gatt->psm < 0)
+		io = bt_io_connect(BT_IO_L2CAP, connect_cb, gatt, NULL, gerr,
+			BT_IO_OPT_SOURCE_BDADDR, &gatt->sba,
+			BT_IO_OPT_DEST_BDADDR, &gatt->dba,
+			BT_IO_OPT_CID, GATT_CID,
+			BT_IO_OPT_SEC_LEVEL, BT_IO_SEC_LOW,
+			BT_IO_OPT_INVALID);
+	else
+		io = bt_io_connect(BT_IO_L2CAP, connect_cb, gatt, NULL, gerr,
 			BT_IO_OPT_SOURCE_BDADDR, &gatt->sba,
 			BT_IO_OPT_DEST_BDADDR, &gatt->dba,
 			BT_IO_OPT_PSM, gatt->psm,
@@ -466,14 +403,11 @@ static DBusMessage *register_watcher(DBusConnection *conn,
 
 	if (!dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
 							DBUS_TYPE_INVALID))
-		return invalid_args(msg);
+		return btd_error_invalid_args(msg);
 
 	if (l2cap_connect(prim->gatt, &gerr, TRUE) < 0) {
-		DBusMessage *reply;
-		reply = g_dbus_create_error(msg, ERROR_INTERFACE ".Failed",
-							"%s", gerr->message);
+		DBusMessage *reply = btd_error_failed(msg, gerr->message);
 		g_error_free(gerr);
-
 		return reply;
 	}
 
@@ -500,7 +434,7 @@ static DBusMessage *unregister_watcher(DBusConnection *conn,
 
 	if (!dbus_message_get_args(msg, NULL, DBUS_TYPE_OBJECT_PATH, &path,
 							DBUS_TYPE_INVALID))
-		return invalid_args(msg);
+		return btd_error_invalid_args(msg);
 
 	match = g_new0(struct watcher, 1);
 	match->name = g_strdup(sender);
@@ -508,7 +442,7 @@ static DBusMessage *unregister_watcher(DBusConnection *conn,
 	l = g_slist_find_custom(prim->watchers, match, watcher_cmp);
 	watcher_free(match);
 	if (!l)
-		return not_authorized(msg);
+		return btd_error_not_authorized(msg);
 
 	watcher = l->data;
 	g_dbus_remove_watch(conn, watcher->id);
@@ -518,15 +452,6 @@ static DBusMessage *unregister_watcher(DBusConnection *conn,
 	return dbus_message_new_method_return(msg);
 }
 
-static GDBusMethodTable prim_methods[] = {
-	{ "GetCharacteristics",	"",	"a{oa{sv}}", get_characteristics},
-	{ "RegisterCharacteristicsWatcher",	"o", "",
-						register_watcher	},
-	{ "UnregisterCharacteristicsWatcher",	"o", "",
-						unregister_watcher	},
-	{ }
-};
-
 static DBusMessage *set_value(DBusConnection *conn, DBusMessage *msg,
 			DBusMessageIter *iter, struct characteristic *chr)
 {
@@ -538,18 +463,15 @@ static DBusMessage *set_value(DBusConnection *conn, DBusMessage *msg,
 
 	if (dbus_message_iter_get_arg_type(iter) != DBUS_TYPE_ARRAY ||
 			dbus_message_iter_get_element_type(iter) != DBUS_TYPE_BYTE)
-		return invalid_args(msg);
+		return btd_error_invalid_args(msg);
 
 	dbus_message_iter_recurse(iter, &sub);
 
 	dbus_message_iter_get_fixed_array(&sub, &value, &len);
 
 	if (l2cap_connect(gatt, &gerr, FALSE) < 0) {
-		DBusMessage *reply;
-		reply = g_dbus_create_error(msg, ERROR_INTERFACE ".Failed",
-							"%s", gerr->message);
+		DBusMessage *reply = btd_error_failed(msg, gerr->message);
 		g_error_free(gerr);
-
 		return reply;
 	}
 
@@ -587,23 +509,23 @@ static DBusMessage *set_property(DBusConnection *conn,
 	const char *property;
 
 	if (!dbus_message_iter_init(msg, &iter))
-		return invalid_args(msg);
+		return btd_error_invalid_args(msg);
 
 	if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING)
-		return invalid_args(msg);
+		return btd_error_invalid_args(msg);
 
 	dbus_message_iter_get_basic(&iter, &property);
 	dbus_message_iter_next(&iter);
 
 	if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_VARIANT)
-		return invalid_args(msg);
+		return btd_error_invalid_args(msg);
 
 	dbus_message_iter_recurse(&iter, &sub);
 
 	if (g_str_equal("Value", property))
 		return set_value(conn, msg, &sub, chr);
 
-	return invalid_args(msg);
+	return btd_error_invalid_args(msg);
 }
 
 static GDBusMethodTable char_methods[] = {
@@ -613,21 +535,6 @@ static GDBusMethodTable char_methods[] = {
 	{ }
 };
 
-static void register_primary(struct gatt_service *gatt)
-{
-	GSList *l;
-
-	for (l = gatt->primary; l; l = l->next) {
-		struct primary *prim = l->data;
-		g_dbus_register_interface(connection, prim->path,
-				CHAR_INTERFACE, prim_methods,
-				NULL, NULL, prim, NULL);
-		DBG("Registered: %s", prim->path);
-
-		device_add_service(gatt->dev, prim->path);
-	}
-}
-
 static char *characteristic_list_to_string(GSList *chars)
 {
 	GString *characteristics;
@@ -661,10 +568,11 @@ static void store_characteristics(struct gatt_service *gatt,
 							struct primary *prim)
 {
 	char *characteristics;
+	struct att_primary *att = prim->att;
 
 	characteristics = characteristic_list_to_string(prim->chars);
 
-	write_device_characteristics(&gatt->sba, &gatt->dba, prim->start,
+	write_device_characteristics(&gatt->sba, &gatt->dba, att->start,
 							characteristics);
 
 	g_free(characteristics);
@@ -728,6 +636,7 @@ static GSList *string_to_characteristic_list(struct primary *prim,
 static void load_characteristics(gpointer data, gpointer user_data)
 {
 	struct primary *prim = data;
+	struct att_primary *att = prim->att;
 	struct gatt_service *gatt = user_data;
 	GSList *chrs_list;
 	char *str;
@@ -737,7 +646,7 @@ static void load_characteristics(gpointer data, gpointer user_data)
 		return;
 	}
 
-	str = read_device_characteristics(&gatt->sba, &gatt->dba, prim->start);
+	str = read_device_characteristics(&gatt->sba, &gatt->dba, att->start);
 	if (str == NULL)
 		return;
 
@@ -935,6 +844,7 @@ static void char_discovered_cb(guint8 status, const guint8 *pdu, guint16 plen,
 {
 	struct query_data *current = user_data;
 	struct primary *prim = current->prim;
+	struct att_primary *att = prim->att;
 	struct gatt_service *gatt = prim->gatt;
 	struct att_data_list *list;
 	uint16_t last, *previous_end = NULL;
@@ -983,15 +893,15 @@ static void char_discovered_cb(guint8 status, const guint8 *pdu, guint16 plen,
 	}
 
 	if (previous_end)
-		*previous_end = prim->end;
+		*previous_end = att->end;
 
 	att_data_list_free(list);
 
-	if (last >= prim->end)
+	if (last >= att->end)
 		goto done;
 
 	/* Fetch remaining characteristics for the CURRENT primary service */
-	gatt_discover_char(gatt->attrib, last + 1, prim->end,
+	gatt_discover_char(gatt->attrib, last + 1, att->end,
 						char_discovered_cb, current);
 
 	return;
@@ -1007,314 +917,101 @@ fail:
 	g_free(current);
 }
 
-static void *attr_data_from_string(const char *str)
+static DBusMessage *discover_char(DBusConnection *conn, DBusMessage *msg,
+								void *data)
 {
-	uint8_t *data;
-	int size, i;
-	char tmp[3];
-
-	size = strlen(str) / 2;
-	data = g_try_malloc0(size);
-	if (data == NULL)
-		return NULL;
-
-	tmp[2] = '\0';
-	for (i = 0; i < size; i++) {
-		memcpy(tmp, str + (i * 2), 2);
-		data[i] = (uint8_t) strtol(tmp, NULL, 16);
-	}
-
-	return data;
-}
-
-static int find_primary(gconstpointer a, gconstpointer b)
-{
-	const struct primary *primary = a;
-	uint16_t handle = GPOINTER_TO_UINT(b);
-
-	if (handle < primary->start)
-		return -1;
-
-	if (handle > primary->end)
-		return -1;
-
-	return 0;
-}
-
-static int find_characteristic(gconstpointer a, gconstpointer b)
-{
-	const struct characteristic *chr = a;
-	uint16_t handle = GPOINTER_TO_UINT(b);
-
-	if (handle < chr->handle)
-		return -1;
-
-	if (handle > chr->end)
-		return -1;
-
-	return 0;
-}
-
-static void load_attribute_data(char *key, char *value, void *data)
-{
-	struct gatt_service *gatt = data;
-	struct characteristic *chr;
-	struct primary *primary;
-	char addr[18], dst[18];
-	uint16_t handle;
-	uuid_t uuid;
-	GSList *l;
-	guint h;
-
-	if (sscanf(key, "%17s#%04hX", addr, &handle) < 2)
-		return;
-
-	ba2str(&gatt->dba, dst);
-
-	if (strcmp(addr, dst) != 0)
-		return;
-
-	h = handle;
-
-	l = g_slist_find_custom(gatt->primary, GUINT_TO_POINTER(h),
-								find_primary);
-	if (!l)
-		return;
-
-	primary = l->data;
-
-	l = g_slist_find_custom(primary->chars, GUINT_TO_POINTER(h),
-							find_characteristic);
-	if (!l)
-		return;
-
-	chr = l->data;
-
-	/* value[] contains "<UUID>#<data>", but bt_string2uuid() expects a
-	 * string containing only the UUID. To avoid creating a new buffer,
-	 * "truncate" the string in place before calling bt_string2uuid(). */
-	value[MAX_LEN_UUID_STR - 1] = '\0';
-	if (bt_string2uuid(&uuid, value) < 0)
-		return;
-
-	/* Fill the characteristic field according to the attribute type. */
-	if (uuid_desc16_cmp(&uuid, GATT_CHARAC_USER_DESC_UUID) == 0)
-		chr->desc = attr_data_from_string(value + MAX_LEN_UUID_STR);
-	else if (uuid_desc16_cmp(&uuid, GATT_CHARAC_FMT_UUID) == 0)
-		chr->format = attr_data_from_string(value + MAX_LEN_UUID_STR);
-}
-
-static char *primary_list_to_string(GSList *primary_list)
-{
-	GString *services;
-	GSList *l;
-
-	services = g_string_new(NULL);
-
-	for (l = primary_list; l; l = l->next) {
-		struct primary *primary = l->data;
-		uuid_t *uuid128;
-		char service[64];
-		char uuidstr[MAX_LEN_UUID_STR];
-
-		memset(service, 0, sizeof(service));
-
-		uuid128 = sdp_uuid_to_uuid128(&primary->uuid);
-		sdp_uuid2strn(uuid128, uuidstr, MAX_LEN_UUID_STR);
-
-		bt_free(uuid128);
-
-		snprintf(service, sizeof(service), "%04X#%04X#%s ",
-				primary->start, primary->end, uuidstr);
-
-		services = g_string_append(services, service);
-	}
-
-	return g_string_free(services, FALSE);
-}
-
-static GSList *string_to_primary_list(struct gatt_service *gatt,
-							const char *str)
-{
-	GSList *l = NULL;
-	char **services;
-	int i;
-
-	if (str == NULL)
-		return NULL;
-
-	services = g_strsplit(str, " ", 0);
-	if (services == NULL)
-		return NULL;
-
-	for (i = 0; services[i]; i++) {
-		struct primary *prim;
-		char uuidstr[MAX_LEN_UUID_STR + 1];
-		int ret;
-
-		prim = g_new0(struct primary, 1);
-		prim->gatt = gatt;
-
-		ret = sscanf(services[i], "%04hX#%04hX#%s", &prim->start,
-							&prim->end, uuidstr);
-
-		if (ret < 3) {
-			g_free(prim);
-			continue;
-		}
-
-		prim->path = g_strdup_printf("%s/service%04x", gatt->path,
-								prim->start);
-
-		bt_string2uuid(&prim->uuid, uuidstr);
-
-		l = g_slist_append(l, prim);
-	}
-
-	g_strfreev(services);
-
-	return l;
-}
-
-static void store_primary_services(struct gatt_service *gatt)
-{
-       char *services;
-
-       services = primary_list_to_string(gatt->primary);
-
-       write_device_services(&gatt->sba, &gatt->dba, services);
-
-       g_free(services);
-}
-
-static gboolean load_primary_services(struct gatt_service *gatt)
-{
-	GSList *primary_list;
-	char *str;
-
-	if (gatt->primary) {
-		DBG("Services already loaded");
-		return FALSE;
-	}
-
-	str = read_device_services(&gatt->sba, &gatt->dba);
-	if (str == NULL)
-		return FALSE;
-
-	primary_list = string_to_primary_list(gatt, str);
-
-	free(str);
-
-	if (primary_list == NULL)
-		return FALSE;
-
-	gatt->primary = primary_list;
-	register_primary(gatt);
-
-	g_slist_foreach(gatt->primary, load_characteristics, gatt);
-	read_device_attributes(&gatt->sba, load_attribute_data, gatt);
-
-	return TRUE;
-}
-
-static void discover_all_char(gpointer data, gpointer user_data)
-{
-	struct query_data *qchr;
-	struct gatt_service *gatt = user_data;
 	struct primary *prim = data;
+	struct att_primary *att = prim->att;
+	struct gatt_service *gatt = prim->gatt;
+	struct query_data *qchr;
+	GError *gerr = NULL;
+
+	if (l2cap_connect(prim->gatt, &gerr, FALSE) < 0) {
+		DBusMessage *reply = btd_error_failed(msg, gerr->message);
+		g_error_free(gerr);
+		return reply;
+	}
 
 	qchr = g_new0(struct query_data, 1);
 	qchr->prim = prim;
 
-	gatt->attrib = g_attrib_ref(gatt->attrib);
-	gatt_discover_char(gatt->attrib, prim->start, prim->end,
+	gatt_discover_char(gatt->attrib, att->start, att->end,
 						char_discovered_cb, qchr);
+
+	return dbus_message_new_method_return(msg);
 }
 
-static void primary_cb(guint8 status, const guint8 *pdu, guint16 plen,
-							gpointer user_data)
+static DBusMessage *prim_get_properties(DBusConnection *conn, DBusMessage *msg,
+								void *data)
 {
-	struct gatt_service *gatt = user_data;
-	struct att_data_list *list;
-	unsigned int i;
-	uint16_t end, start;
+	struct primary *prim = data;
+	DBusMessage *reply;
+	DBusMessageIter iter;
+	DBusMessageIter dict;
+	GSList *l;
+	char **chars;
+	int i;
 
-	if (status == ATT_ECODE_ATTR_NOT_FOUND) {
-		if (gatt->primary == NULL)
-			goto done;
+	reply = dbus_message_new_method_return(msg);
+	if (!reply)
+		return NULL;
 
-		store_primary_services(gatt);
-		register_primary(gatt);
+	dbus_message_iter_init_append(reply, &iter);
 
-		g_slist_foreach(gatt->primary, discover_all_char, gatt);
-		goto done;
+	dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
+			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
+			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
+			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
+
+	chars = g_new0(char *, g_slist_length(prim->chars) + 1);
+
+	for (i = 0, l = prim->chars; l; l = l->next, i++) {
+		struct characteristic *chr = l->data;
+		chars[i] = chr->path;
 	}
 
-	if (status != 0) {
-		error("Discover all primary services failed: %s",
-						att_ecode2str(status));
-		goto done;
-	}
+	dict_append_array(&dict, "Characteristics", DBUS_TYPE_OBJECT_PATH,
+								&chars, i);
+	g_free(chars);
 
-	list = dec_read_by_grp_resp(pdu, plen);
-	if (list == NULL) {
-		error("Protocol error");
-		goto done;
-	}
+	dbus_message_iter_close_container(&iter, &dict);
 
-	DBG("Read by Group Type Response received");
+	return reply;
+}
 
-	for (i = 0, end = 0; i < list->num; i++) {
+static GDBusMethodTable prim_methods[] = {
+	{ "Discover",		"",	"",		discover_char	},
+	{ "RegisterCharacteristicsWatcher",	"o", "",
+						register_watcher	},
+	{ "UnregisterCharacteristicsWatcher",	"o", "",
+						unregister_watcher	},
+	{ "GetProperties",	"",	"a{sv}",prim_get_properties	},
+	{ }
+};
+
+static void register_primaries(struct gatt_service *gatt, GSList *primaries)
+{
+	GSList *l;
+
+	for (l = primaries; l; l = l->next) {
+		struct att_primary *att = l->data;
 		struct primary *prim;
-		uint8_t *info = list->data[i];
-
-		/* Each element contains: attribute handle, end group handle
-		 * and attribute value */
-		start = att_get_u16(info);
-		end = att_get_u16(&info[2]);
 
 		prim = g_new0(struct primary, 1);
+		prim->att = att;
 		prim->gatt = gatt;
-		prim->start = start;
-		prim->end = end;
-
-		if (list->len == 6) {
-			sdp_uuid16_create(&prim->uuid,
-					att_get_u16(&info[4]));
-
-		} else if (list->len == 20) {
-			/* FIXME: endianness */
-			sdp_uuid128_create(&prim->uuid, &info[4]);
-		} else {
-			DBG("ATT: Invalid Length field");
-			g_free(prim);
-			att_data_list_free(list);
-			goto done;
-		}
-
 		prim->path = g_strdup_printf("%s/service%04x", gatt->path,
-								prim->start);
+								att->start);
+
+		g_dbus_register_interface(connection, prim->path,
+				CHAR_INTERFACE, prim_methods,
+				NULL, NULL, prim, NULL);
+		DBG("Registered: %s", prim->path);
 
 		gatt->primary = g_slist_append(gatt->primary, prim);
+		btd_device_add_service(gatt->dev, prim->path);
+		load_characteristics(prim, gatt);
 	}
-
-	att_data_list_free(list);
-
-	if (end == 0) {
-		DBG("ATT: Invalid PDU format");
-		goto done;
-	}
-
-	/*
-	 * Discover all primary services sub-procedure shall send another
-	 * Read by Group Type Request until Error Response is received and
-	 * the Error Code is set to Attribute Not Found.
-	 */
-	gatt->attrib = g_attrib_ref(gatt->attrib);
-	gatt->atid = gatt_discover_primary(gatt->attrib, end + 1, 0xffff, NULL,
-							primary_cb, gatt);
-done:
-	g_attrib_unref(gatt->attrib);
 }
 
 int attrib_client_register(struct btd_device *device, int psm)
@@ -1322,6 +1019,7 @@ int attrib_client_register(struct btd_device *device, int psm)
 	struct btd_adapter *adapter = device_get_adapter(device);
 	const char *path = device_get_path(device);
 	struct gatt_service *gatt;
+	GSList *primaries = btd_device_get_primaries(device);
 	bdaddr_t sba, dba;
 
 	adapter_get_address(adapter, &sba);
@@ -1335,8 +1033,7 @@ int attrib_client_register(struct btd_device *device, int psm)
 	bacpy(&gatt->dba, &dba);
 	gatt->psm = psm;
 
-	if (load_primary_services(gatt))
-		DBG("Primary services loaded");
+	register_primaries(gatt, primaries);
 
 	gatt_services = g_slist_append(gatt_services, gatt);
 
